const { Request } = require('jest-express/lib/request');
const util = require('./logger');

describe('Utility - logger', () => {
  const req = new Request('/api/status', {
    method: 'PUT'
  });

  let debugSpy;
  let infoSpy;
  let warnSpy;
  let errorSpy;
  let infoSpyLog;
  let errorSpyLog;

  beforeEach(() => {
    debugSpy = jest.spyOn(util, 'debug');
    infoSpy = jest.spyOn(util, 'info');
    infoSpyLog = jest.spyOn(util.logger, 'info');
    warnSpy = jest.spyOn(util, 'warn');
    errorSpy = jest.spyOn(util.logger, 'error');
    errorSpyLog = jest.spyOn(util.logger, 'error');
  });

  afterEach(() => {
    jest.restoreAllMocks();
  });

  test('should write info log stream', () => {
    util.logger.stream.write(req);

    expect(infoSpyLog).toHaveBeenCalledTimes(1);
  });

  it('should write error log stream', () => {
    util.logger.streamError.write(req);

    expect(errorSpyLog).toHaveBeenCalledTimes(1);
  });

  it('should write debug log', () => {
    util.debug('message', { data: 1 });

    expect(debugSpy).toHaveBeenCalledTimes(1);
  });

  it('should write warn log', () => {
    util.warn('message', { data: 1 });

    expect(warnSpy).toHaveBeenCalledTimes(1);
  });

  it('should write info log', () => {
    util.info('message', { data: 1 });

    expect(infoSpy).toHaveBeenCalledTimes(1);
  });

  it('should capture error response', () => {
    const methodName = 'method';
    const errorMessage = 'error';
    const response = 'stupid';

    const error = {
      message: errorMessage,
      response: {
        data: response
      }
    };
    util.captureError('title', error, methodName);

    expect(errorSpy).toHaveBeenCalledTimes(1);
  });

  it('should capture error response with stack', () => {
    const methodName = 'method';
    const errorMessage = 'error';
    const response = 'stupid';
    const stack = 'stack';

    const error = {
      message: errorMessage,
      response: {
        data: response
      },
      stack
    };
    util.captureError('title', error, methodName);

    expect(errorSpy).toHaveBeenCalledTimes(1);
  });
});
